@@ -17,37 +17,18 @@ TODO
 """
 __all__ = ['SelectorList']
 __docformat__ = 'restructuredtext'
-__version__ = '$Id: selectorlist.py 1174 2008-03-20 17:43:07Z cthedot $'
+__version__ = '$Id$'
 
-import xml.dom
-import cssutils
 from selector import Selector
+import cssutils
+import xml.dom
 
 class SelectorList(cssutils.util.Base, cssutils.util.ListSeq):
-    """
-    (cssutils) a list of Selectors of a CSSStyleRule
-
-    Properties
-    ==========
-    length: of type unsigned long, readonly
-        The number of Selector elements in the list.
-    parentRule: of type CSSRule, readonly
-        The CSS rule that contains this selector list or None if this
-        list is not attached to a CSSRule.
-    selectorText: of type DOMString
-        The textual representation of the selector for the rule set. The
-        implementation may have stripped out insignificant whitespace while
-        parsing the selector.
-    seq: (internal use!)
-        A list of Selector objects
-    wellformed
-        if this selectorlist is wellformed regarding the Selector spec
-    """
+    """A list of :class:`~cssutils.css.Selector` objects
+    of a :class:`~cssutils.css.CSSStyleRule`."""
     def __init__(self, selectorText=None, parentRule=None,
                  readonly=False):
         """
-        initializes SelectorList with optional selectorText
-
         :Parameters:
             selectorText
                 parsable list of Selectors
@@ -63,21 +44,23 @@ class SelectorList(cssutils.util.Base, cssutils.util.ListSeq):
 
         self._readonly = readonly
 
-    def __prepareset(self, newSelector, namespaces=None):
-        "used by appendSelector and __setitem__"
-        if not namespaces:
-            namespaces = {}
-        self._checkReadonly()
-        if not isinstance(newSelector, Selector):
-            newSelector = Selector((newSelector, namespaces),
-                                   parentList=self)
-        if newSelector.wellformed:
-            newSelector._parent = self # maybe set twice but must be!
-            return newSelector
+    def __repr__(self):
+        if self._namespaces:
+            st = (self.selectorText, self._namespaces)
+        else:
+            st = self.selectorText
+        return u"cssutils.css.%s(selectorText=%r)" % (self.__class__.__name__, 
+                                                      st)
+
+    def __str__(self):
+        return u"<cssutils.css.%s object selectorText=%r _namespaces=%r at " \
+               u"0x%x>" % (self.__class__.__name__,
+                           self.selectorText,
+                           self._namespaces,
+                           id(self))
 
     def __setitem__(self, index, newSelector):
-        """
-        overwrites ListSeq.__setitem__
+        """Overwrite ListSeq.__setitem__
 
         Any duplicate Selectors are **not** removed.
         """
@@ -85,16 +68,22 @@ class SelectorList(cssutils.util.Base, cssutils.util.ListSeq):
         if newSelector:
             self.seq[index] = newSelector
 
-    def append(self, newSelector):
-        "same as appendSelector(newSelector)"
-        self.appendSelector(newSelector)
-
-    length = property(lambda self: len(self),
-        doc="The number of Selector elements in the list.")
-
+    def __prepareset(self, newSelector, namespaces=None):
+        "Used by appendSelector and __setitem__"
+        if not namespaces:
+            namespaces = {}
+        self._checkReadonly()
+        if not isinstance(newSelector, Selector):
+            newSelector = Selector((newSelector, namespaces),
+                                   parent=self)
+        if newSelector.wellformed:
+            newSelector._parent = self # maybe set twice but must be!
+            return newSelector
 
     def __getNamespaces(self):
-        "uses children namespaces if not attached to a sheet, else the sheet's ones"
+        """Use children namespaces if not attached to a sheet, else the sheet's
+        ones.
+        """
         try:
             return self.parentRule.parentStyleSheet.namespaces
         except AttributeError:
@@ -103,17 +92,67 @@ class SelectorList(cssutils.util.Base, cssutils.util.ListSeq):
                 namespaces.update(selector._namespaces)
             return namespaces
 
-    _namespaces = property(__getNamespaces, doc="""if this SelectorList is
+    def _getUsedUris(self):
+        "Used by CSSStyleSheet to check if @namespace rules are needed"
+        uris = set()
+        for s in self:
+            uris.update(s._getUsedUris())
+        return uris
+
+    _namespaces = property(__getNamespaces, doc="""If this SelectorList is
         attached to a CSSStyleSheet the namespaces of that sheet are mirrored
         here. While the SelectorList (or parentRule(s) are
         not attached the namespaces of all children Selectors are used.""")
 
-    parentRule = property(lambda self: self._parentRule,
-        doc="(DOM) The CSS rule that contains this SelectorList or\
-        None if this SelectorList is not attached to a CSSRule.")
+    def append(self, newSelector):
+        "Same as :meth:`appendSelector`."
+        self.appendSelector(newSelector)
+
+    def appendSelector(self, newSelector):
+        """
+        Append `newSelector` to this list (a string will be converted to a 
+        :class:`~cssutils.css.Selector`).
+
+        :param newSelector:
+            comma-separated list of selectors (as a single string) or a tuple of
+            `(newSelector, dict-of-namespaces)`
+        :returns: New :class:`~cssutils.css.Selector` or ``None`` if 
+            `newSelector` is not wellformed.
+        :exceptions:
+            - :exc:`~xml.dom.NamespaceErr`:
+              Raised if the specified selector uses an unknown namespace
+              prefix.
+            - :exc:`~xml.dom.SyntaxErr`:
+              Raised if the specified CSS string value has a syntax error
+              and is unparsable.
+            - :exc:`~xml.dom.NoModificationAllowedErr`:
+              Raised if this rule is readonly.
+        """
+        self._checkReadonly()
+
+        # might be (selectorText, namespaces)
+        newSelector, namespaces = self._splitNamespacesOff(newSelector)
+        try:
+            # use parent's only if available
+            namespaces = self.parentRule.parentStyleSheet.namespaces
+        except AttributeError:
+            # use already present namespaces plus new given ones
+            _namespaces = self._namespaces
+            _namespaces.update(namespaces)
+            namespaces = _namespaces
+
+        newSelector = self.__prepareset(newSelector, namespaces)
+        if newSelector:
+            seq = self.seq[:]
+            del self.seq[:]
+            for s in seq:
+                if s.selectorText != newSelector.selectorText:
+                    self.seq.append(s)
+            self.seq.append(newSelector)
+            return newSelector
 
     def _getSelectorText(self):
-        "returns serialized format"
+        "Return serialized format."
         return cssutils.ser.do_css_SelectorList(self)
 
     def _setSelectorText(self, selectorText):
@@ -121,14 +160,14 @@ class SelectorList(cssutils.util.Base, cssutils.util.ListSeq):
         :param selectorText:
             comma-separated list of selectors or a tuple of
             (selectorText, dict-of-namespaces)
-        :Exceptions:
-            - `NAMESPACE_ERR`: (Selector)
+        :exceptions:
+            - :exc:`~xml.dom.NamespaceErr`:
               Raised if the specified selector uses an unknown namespace
               prefix.
-            - `SYNTAX_ERR`: (self)
+            - :exc:`~xml.dom.SyntaxErr`:
               Raised if the specified CSS string value has a syntax error
               and is unparsable.
-            - `NO_MODIFICATION_ALLOWED_ERR`: (self)
+            - :exc:`~xml.dom.NoModificationAllowedErr`:
               Raised if this rule is readonly.
         """
         self._checkReadonly()
@@ -156,7 +195,7 @@ class SelectorList(cssutils.util.Base, cssutils.util.ListSeq):
                     expected = None
 
                 selector = Selector((selectortokens, namespaces),
-                                    parentList=self)
+                                    parent=self)
                 if selector.wellformed:
                     newseq.append(selector)
                 else:
@@ -177,73 +216,19 @@ class SelectorList(cssutils.util.Base, cssutils.util.ListSeq):
                             self._valuestr(selectorText))
         if wellformed:
             self.seq = newseq
-#            for selector in newseq:
-#                 self.appendSelector(selector)
 
     selectorText = property(_getSelectorText, _setSelectorText,
-        doc="""(cssutils) The textual representation of the selector for
-            a rule set.""")
+                            doc=u"(cssutils) The textual representation of the "
+                                u"selector for a rule set.")
+
+    length = property(lambda self: len(self),
+                      doc=u"The number of :class:`~cssutils.css.Selector` "
+                          u"objects in the list.")
+
+    parentRule = property(lambda self: self._parentRule,
+                          doc=u"(DOM) The CSS rule that contains this "
+                              u"SelectorList or ``None`` if this SelectorList "
+                              u"is not attached to a CSSRule.")
 
     wellformed = property(lambda self: bool(len(self.seq)))
 
-    def appendSelector(self, newSelector):
-        """
-        Append newSelector (a string will be converted to a new
-        Selector).
-
-        :param newSelector:
-            comma-separated list of selectors or a tuple of
-            (selectorText, dict-of-namespaces)
-        :returns: New Selector or None if newSelector is not wellformed.
-        :Exceptions:
-            - `NAMESPACE_ERR`: (self)
-              Raised if the specified selector uses an unknown namespace
-              prefix.
-            - `SYNTAX_ERR`: (self)
-              Raised if the specified CSS string value has a syntax error
-              and is unparsable.
-            - `NO_MODIFICATION_ALLOWED_ERR`: (self)
-              Raised if this rule is readonly.
-        """
-        self._checkReadonly()
-
-        # might be (selectorText, namespaces)
-        newSelector, namespaces = self._splitNamespacesOff(newSelector)
-        try:
-            # use parent's only if available
-            namespaces = self.parentRule.parentStyleSheet.namespaces
-        except AttributeError:
-            # use already present namespaces plus new given ones
-            _namespaces = self._namespaces
-            _namespaces.update(namespaces)
-            namespaces = _namespaces
-
-        newSelector = self.__prepareset(newSelector, namespaces)
-        if newSelector:
-            seq = self.seq[:]
-            del self.seq[:]
-            for s in seq:
-                if s.selectorText != newSelector.selectorText:
-                    self.seq.append(s)
-            self.seq.append(newSelector)
-            return newSelector
-
-    def __repr__(self):
-        if self._namespaces:
-            st = (self.selectorText, self._namespaces)
-        else:
-            st = self.selectorText
-        return "cssutils.css.%s(selectorText=%r)" % (
-                self.__class__.__name__, st)
-
-    def __str__(self):
-        return "<cssutils.css.%s object selectorText=%r _namespaces=%r at 0x%x>" % (
-                self.__class__.__name__, self.selectorText, self._namespaces,
-                id(self))
-
-    def _getUsedUris(self):
-        "used by CSSStyleSheet to check if @namespace rules are needed"
-        uris = set()
-        for s in self:
-            uris.update(s._getUsedUris())
-        return uris
