@@ -48,7 +48,7 @@ class Preferences(object):
     indent = 4 * ' '
         Indentation of e.g Properties inside a CSSStyleDeclaration
     indentClosingBrace = True
-        Defines if closing brace of block is indented to match indentation 
+        Defines if closing brace of block is indented to match indentation
         of the block (default) oder match indentation of selector.
     indentSpecificities = False (**EXPERIMENTAL**)
         Indent rules with subset of Selectors and higher Specitivity
@@ -152,7 +152,7 @@ class Preferences(object):
         """Set options resulting in a minified stylesheet.
 
         You may want to set preferences with this convenience method
-        and set settings you want adjusted afterwards.
+        and override specific settings you want adjusted afterwards.
         """
         self.importHrefFormat = 'string'
         self.indent = u''
@@ -183,8 +183,7 @@ class Out(object):
             # remove trailing S
             del self.out[-1]
 
-    def append(self, val, typ=None, space=True, keepS=False, indent=False,
-               lineSeparator=False):
+    def append(self, val, type_=None, space=True, keepS=False, indent=False):
         """Appends val. Adds a single S after each token except as follows:
 
         - typ COMMENT
@@ -199,7 +198,7 @@ class Out(object):
             calls helper.uri
         - val ``{``
             adds LF after
-        - val ``;``
+        - val ``;``, typ 'styletext'
             removes S before and adds LF after
         - val ``, :``
             removes S before
@@ -209,42 +208,43 @@ class Out(object):
             add ``*spacer`` except ``space=False``
         """
         prefspace = self.ser.prefs.spacer
-        if val or typ in ('STRING', 'URI'):
+        if val or type_ in ('STRING', 'URI'):
             # PRE
-            if 'COMMENT' == typ:
+            if 'COMMENT' == type_:
                 if self.ser.prefs.keepComments:
                     val = val.cssText
                 else:
                     return
             elif hasattr(val, 'cssText'):
                 val = val.cssText
-#            elif typ in ('Property', cssutils.css.CSSRule.UNKNOWN_RULE):
+#            elif type_ in ('Property', cssutils.css.CSSRule.UNKNOWN_RULE):
 #                val = val.cssText
-            elif 'S' == typ and not keepS:
+            elif 'S' == type_ and not keepS:
                 return
-            elif 'S' == typ and keepS:
+            elif 'S' == type_ and keepS:
                 val = u' '
-#            elif typ in ('NUMBER', 'DIMENSION', 'PERCENTAGE') and val == u'0':
+#            elif type_ in ('NUMBER', 'DIMENSION', 'PERCENTAGE') and val == u'0':
 #                # remove sign + or - if value is zero
 #                # TODO: only for lenghts!
 #                if self.out and self.out[-1] in u'+-':
 #                    del self.out[-1]
-            elif 'STRING' == typ:
+            elif 'STRING' == type_:
                 # may be empty but MUST not be None
                 if val is None:
                     return
                 val = helper.string(val)
                 if not prefspace:
                     self._remove_last_if_S()
-            elif 'URI' == typ:
+            elif 'URI' == type_:
                 val = helper.uri(val)
-            elif 'HASH' == typ:
+            elif 'HASH' == type_:
                 val = self.ser._hash(val)
-            elif val in u'+>~,:{;)]/=':
+            elif val in u'+>~,:{;)]/=}':
                 self._remove_last_if_S()
 
             # APPEND
-            if indent:
+
+            if indent or (val == u'}' and self.ser.prefs.indentClosingBrace):
                 self.out.append(self.ser._indentblock(val, self.ser._level+1))
             else:
                 if val.endswith(u' '):
@@ -252,10 +252,7 @@ class Out(object):
                 self.out.append(val)
 
             # POST
-            if lineSeparator:
-                # Property , ...
-                pass
-            elif val in u'+>~': # enclose selector combinator
+            if val in u'+>~': # enclose selector combinator
                 self.out.insert(-1, self.ser.prefs.selectorCombinatorSpacer)
                 self.out.append(self.ser.prefs.selectorCombinatorSpacer)
             elif u')' == val and not keepS: # CHAR funcend
@@ -268,11 +265,11 @@ class Out(object):
             elif u'{' == val: # block start
                 self.out.insert(-1, self.ser.prefs.paranthesisSpacer)
                 self.out.append(self.ser.prefs.lineSeparator)
-            elif u';' == val: # end or prop or block
+            elif u';' == val or 'styletext' == type_: # end or prop or block
                 self.out.append(self.ser.prefs.lineSeparator)
-            elif val not in u'}[]()/=' and typ != 'FUNCTION' and space:
+            elif val not in u'}[]()/=' and space and type_ != 'FUNCTION':
                 self.out.append(self.ser.prefs.spacer)
-                if typ != 'STRING' and not self.ser.prefs.spacer and \
+                if type_ != 'STRING' and not self.ser.prefs.spacer and \
                    self.out and not self.out[-1].endswith(u' '):
                     self.out.append(u' ')
 
@@ -305,12 +302,12 @@ class CSSSerializer(object):
         self._selectors = [] # holds SelectorList
         self._selectorlevel = 0 # current specificity nesting level
 
-    def _atkeyword(self, rule, default):
+    def _atkeyword(self, rule):
         "returns default or source atkeyword depending on prefs"
         if self.prefs.defaultAtKeyword:
-            return default
+            return rule.atkeyword # default
         else:
-            return rule.atkeyword
+            return rule._keyword
 
     def _indentblock(self, text, level):
         """
@@ -423,7 +420,7 @@ class CSSSerializer(object):
 
         if variablesText and rule.wellformed and not self.prefs.resolveVariables:
             out = Out(self)
-            out.append(self._atkeyword(rule, u'@variables'))
+            out.append(self._atkeyword(rule))
             for item in rule.seq:
                 # assume comments {
                 out.append(item.value, item.type)
@@ -447,7 +444,7 @@ class CSSSerializer(object):
 
         if styleText and rule.wellformed:
             out = Out(self)
-            out.append(self._atkeyword(rule, u'@font-face'))
+            out.append(self._atkeyword(rule))
             for item in rule.seq:
                 # assume comments {
                 out.append(item.value, item.type)
@@ -473,11 +470,11 @@ class CSSSerializer(object):
         """
         if rule.wellformed:
             out = Out(self)
-            out.append(self._atkeyword(rule, u'@import'))
+            out.append(self._atkeyword(rule))
 
             for item in rule.seq:
-                typ, val = item.type, item.value
-                if 'href' == typ:
+                type_, val = item.type, item.value
+                if 'href' == type_:
                     # "href" or url(href)
                     if self.prefs.importHrefFormat == 'string' or (
                              self.prefs.importHrefFormat != 'uri' and
@@ -485,15 +482,15 @@ class CSSSerializer(object):
                         out.append(val, 'STRING')
                     else:
                         out.append(val, 'URI')
-                elif 'media' == typ:
+                elif 'media' == type_:
                     # media
                     mediaText = self.do_stylesheets_medialist(val)
                     if mediaText and mediaText != u'all':
                         out.append(mediaText)
-                elif 'name' == typ:
+                elif 'name' == type_:
                     out.append(val, 'STRING')
                 else:
-                    out.append(val, typ)
+                    out.append(val, type_)
 
             return out.value(end=u';')
         else:
@@ -512,13 +509,13 @@ class CSSSerializer(object):
         """
         if rule.wellformed:
             out = Out(self)
-            out.append(self._atkeyword(rule, u'@namespace'))
+            out.append(self._atkeyword(rule))
             for item in rule.seq:
-                typ, val = item.type, item.value
-                if 'namespaceURI' == typ:
+                type_, val = item.type, item.value
+                if 'namespaceURI' == type_:
                     out.append(val, 'STRING')
                 else:
-                    out.append(val, typ)
+                    out.append(val, type_)
 
             return out.value(end=u';')
         else:
@@ -537,7 +534,7 @@ class CSSSerializer(object):
             return u''
 
         # @media
-        out = [self._atkeyword(rule, u'@media')]
+        out = [self._atkeyword(rule)]
         if not len(self.prefs.spacer):
             # for now always with space as only webkit supports @mediaall?
             out.append(u' ')
@@ -573,7 +570,7 @@ class CSSSerializer(object):
         out.extend(rulesout)
 
         #     }
-        out.append(u'%s}' % ((self._level + int(self.prefs.indentClosingBrace)) 
+        out.append(u'%s}' % ((self._level + int(self.prefs.indentClosingBrace))
                              * self.prefs.indent))
 
         return u''.join(out)
@@ -586,17 +583,47 @@ class CSSSerializer(object):
             string
         style
             CSSStyleDeclaration
+        cssRules
+            CSSRuleList of MarginRule objects
 
         + CSSComments
         """
-        styleText = self.do_css_CSSStyleDeclaration(rule.style)
-        if styleText and rule.wellformed:
+        # rules
+        rules = u''
+        rulesout = []
+        for r in rule.cssRules:
+            rtext = r.cssText
+            if rtext:
+                rulesout.append(rtext)
+                rulesout.append(self.prefs.lineSeparator)
+
+        rulesText = u''.join(rulesout)#.strip()
+
+        # omit semicolon only if no MarginRules
+        styleText = self.do_css_CSSStyleDeclaration(rule.style,
+                                                    omit=not rulesText)
+
+        if (styleText or rulesText) and rule.wellformed:
             out = Out(self)
-            out.append(self._atkeyword(rule, u'@page'))
+            out.append(self._atkeyword(rule))
             out.append(rule.selectorText)
             out.append(u'{')
-            out.append(u'%s%s}' % (styleText, self.prefs.lineSeparator),
-                       indent=1)
+
+            if styleText:
+                if not rulesText:
+                    out.append(u'%s%s' % (styleText,
+                                          self.prefs.lineSeparator
+                                          ), indent=1)
+                else:
+                    out.append(styleText, type_='styletext', indent=1, space=False)
+
+            if rulesText:
+                out.append(rulesText, indent=1)
+            #?
+            self._level -= 1
+            out.append(u'}')
+            self._level += 1
+
             return out.value()
         else:
             return u''
@@ -611,6 +638,44 @@ class CSSSerializer(object):
                 out.append(item.value, item.type)
         return out.value()
 
+    def do_MarginRule(self, rule):
+        """
+        serializes MarginRule
+
+        atkeyword
+            string
+        style
+            CSSStyleDeclaration
+
+        + CSSComments
+        """
+        # might not be set at all?!
+        if rule.atkeyword:
+            styleText = self.do_css_CSSStyleDeclaration(rule.style)
+
+            if styleText and rule.wellformed:
+                out = Out(self)
+
+#                # use seq but styledecl missing
+#                for item in rule.seq:
+#                    if item.type == 'ATKEYWORD':
+#                        # move logic to Out
+#                        out.append(self._atkeyword(rule), type_=item.type)
+#                    else:
+#                        print type_, val
+#                        out.append(item.value, item.type)
+#                return out.value()
+
+                # ok for now:
+                out.append(self._atkeyword(rule), type_='ATKEYWORD')
+                out.append(u'{')
+                out.append(u'%s%s' % (self._indentblock(styleText, self._level+1),
+                                       self.prefs.lineSeparator))
+                out.append(u'}')
+                return out.value()
+
+        return u''
+
     def do_CSSUnknownRule(self, rule):
         """
         serializes CSSUnknownRule
@@ -623,7 +688,7 @@ class CSSSerializer(object):
 
             stacks = []
             for item in rule.seq:
-                typ, val = item.type, item.value
+                type_, val = item.type, item.value
                 # PRE
                 if u'}' == val:
                     # close last open item on stack
@@ -636,9 +701,9 @@ class CSSSerializer(object):
                         val = self._indentblock(val, min(1, len(stacks)+1))
                 # APPEND
                 if stacks:
-                    stacks[-1].append(val, typ)
+                    stacks[-1].append(val, type_)
                 else:
-                    out.append(val, typ)
+                    out.append(val, type_)
 
                 # POST
                 if u'{' == val:
@@ -704,7 +769,7 @@ class CSSSerializer(object):
                     self.prefs.lineSeparator,
                     self._indentblock(styleText, self._level + 1),
                     self.prefs.lineSeparator,
-                    (self._level + int(self.prefs.indentClosingBrace)) 
+                    (self._level + int(self.prefs.indentClosingBrace))
                     * self.prefs.indent),
                 self._selectorlevel)
 
@@ -739,13 +804,13 @@ class CSSSerializer(object):
 
             DEFAULTURI = selector._namespaces.get('', None)
             for item in selector.seq:
-                typ, val = item.type, item.value
+                type_, val = item.type, item.value
                 if isinstance(val, tuple):
                     # namespaceURI|name (element or attribute)
                     namespaceURI, name = val
                     if DEFAULTURI == namespaceURI or (not DEFAULTURI and
                                                       namespaceURI is None):
-                        out.append(name, typ, space=False)
+                        out.append(name, type_, space=False)
                     else:
                         if namespaceURI == cssutils._ANYNS:
                             prefix = u'*'
@@ -756,9 +821,9 @@ class CSSSerializer(object):
                             except IndexError:
                                 prefix = u''
 
-                        out.append(u'%s|%s' % (prefix, name), typ, space=False)
+                        out.append(u'%s|%s' % (prefix, name), type_, space=False)
                 else:
-                    out.append(val, typ, space=False, keepS=True)
+                    out.append(val, type_, space=False, keepS=True)
 
             return out.value()
         else:
@@ -795,7 +860,7 @@ class CSSSerializer(object):
         else:
             return u''
 
-    def do_css_CSSStyleDeclaration(self, style, separator=None):
+    def do_css_CSSStyleDeclaration(self, style, separator=None, omit=True):
         """
         Style declaration of CSSStyleRule
         """
@@ -818,8 +883,10 @@ class CSSSerializer(object):
                          or not isinstance(item.value, cssutils.css.Property)]
 
             out = []
+            omitLastSemicolon = omit and self.prefs.omitLastSemicolon
+
             for i, item in enumerate(seq):
-                typ, val = item.type, item.value
+                type_, val = item.type, item.value
                 if isinstance(val, cssutils.css.CSSComment):
                     # CSSComment
                     if self.prefs.keepComments:
@@ -829,7 +896,7 @@ class CSSSerializer(object):
                     # PropertySimilarNameList
                     if val.cssText:
                         out.append(val.cssText)
-                        if not (self.prefs.omitLastSemicolon and i==len(seq)-1):
+                        if not (omitLastSemicolon and i==len(seq)-1):
                             out.append(u';')
                         out.append(separator)
                 elif isinstance(val, cssutils.css.CSSUnknownRule):
@@ -931,6 +998,12 @@ class CSSSerializer(object):
 
             return out.value()
 
+    def _strip_zeros(self, s):
+        i = s.index(u'.') + 2
+        a, b = s[0:i], s[i:len(s)]
+        b = b.rstrip('0')
+        return a + b
+
     def do_css_Value(self, value, valuesOnly=None):
         """Serializes a Value, valuesOnly is ignored"""
         if not value:
@@ -948,28 +1021,29 @@ class CSSSerializer(object):
                     # cut off after . which is zero anyway
                     val = unicode(int(value.value))
                 elif self.prefs.omitLeadingZero and -1 < value.value < 1:
-                    v = unicode(value.value)
+                    v = self._strip_zeros(u'%f' % value.value) # issue #27
                     val = v
                     if value._sign == u'-':
                         val = v[0] + v[2:]
                     else:
                         val = v[1:]
                 else:
-                    val = unicode(value.value)
+
+                    val = self._strip_zeros(u'%f' % value.value) # issue #27
 
                 # keep '+' if given
                 if value.value != 0 and value._sign == u'+':
                     sign = u'+'
                 else:
                     sign = u''
-                    
+
                 out.append(sign + val + dim, value.type)
-        
+
             else:
                 # e.g. URI
                 out.append(value.value, value.type)
-                
-        return out.value()            
+
+        return out.value()
 
     def do_css_ColorValue(self, value, valuesOnly=False):
         """Serialize a ColorValue, a HASH simple value or FUNCTION"""
@@ -977,7 +1051,7 @@ class CSSSerializer(object):
             return {'FUNCTION': self.do_css_CSSFunction,
                     'HASH': self.do_css_Value,
                     'IDENT': self.do_css_Value
-                    }[value.colorType](value, 
+                    }[value.colorType](value,
                                        valuesOnly=valuesOnly)
         except KeyError, e:
             return u''
