@@ -1,17 +1,17 @@
+# -*- coding: utf-8 -*-
 #!/usr/bin/env python
 """encutils - encoding detection collection for Python
 
-encutils
-========
+:Version: 0.9.8
 :Author: Christof Hoeke, see http://cthedot.de/encutils/
-:Copyright: 2005-2008: Christof Hoeke
+:Contributor: Robert Siemer, Fredrik Hedman <fredrik.hedman@me.com> ported to python3
+:Copyright: 2005-2012: Christof Hoeke
 :License: encutils has a dual-license, please choose whatever you prefer:
 
-    * encutils is published under the `LGPL 3 or later <http://cthedot.de/encutils/license/>`__
-    * encutils is published under the  
+    * encutils is published under the
+      `LGPL 3 or later <http://cthedot.de/encutils/license/>`__
+    * encutils is published under the
       `Creative Commons License <http://creativecommons.org/licenses/by/3.0/>`__.
-      
-    This file is part of encutils.
 
     encutils is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -25,65 +25,45 @@ encutils
 
     You should have received a copy of the GNU Lesser General Public License
     along with encutils.  If not, see <http://www.gnu.org/licenses/>.
- 
+
 
 A collection of helper functions to detect encodings of text files (like HTML, XHTML, XML, CSS, etc.) retrieved via HTTP, file or string.
 
-``getEncodingInfo`` is probably the main function of interest which uses
+:func:`getEncodingInfo` is probably the main function of interest which uses
 other supplied functions itself and gathers all information together and
-supplies an ``EncodingInfo`` object with the following properties:
-
-- ``encoding``: The guessed encoding
-    Encoding is the explicit or implicit encoding or None and
-    always lowercase.
-
-- from HTTP response    
-    * ``http_encoding``
-    * ``http_media_type``
-
-- from HTML <meta> element    
-    * ``meta_encoding``
-    * ``meta_media_type``
-
-- from XML declaration
-    * ``xml_encoding``
+supplies an :class:`EncodingInfo` object.
 
 example::
 
     >>> import encutils
     >>> info = encutils.getEncodingInfo(url='http://cthedot.de/encutils/')
-    
-    >>> print info  # = str(info)
+
+    >>> str(info)
     utf-8
-    
-    >>> info        # = repr(info)
-    <encutils.EncodingInfo object encoding='utf-8' mismatch=False at 0xb86d30>
-    
-    >>> print info.logtext
+
+    >>> repr(info) # doctest:+ELLIPSIS
+    <encutils.EncodingInfo object encoding='utf-8' mismatch=False at...>
+
+    >>> info.logtext
     HTTP media_type: text/html
     HTTP encoding: utf-8
-    HTML META media_type: text/html
-    HTML META encoding: utf-8
     Encoding (probably): utf-8 (Mismatch: False)
-
+    <BLANKLINE>
 
 references
-==========
-XML
-    RFC 3023 (http://www.ietf.org/rfc/rfc3023.txt)
-    
-    easier explained in 
-        - http://feedparser.org/docs/advanced.html
-        - http://www.xml.com/pub/a/2004/07/21/dive.html
-        
-HTML
-    http://www.w3.org/TR/REC-html40/charset.html#h-5.2.2
+    XML
+        RFC 3023 (http://www.ietf.org/rfc/rfc3023.txt)
+
+        easier explained in
+            - http://feedparser.org/docs/advanced.html
+            - http://www.xml.com/pub/a/2004/07/21/dive.html
+
+    HTML
+        http://www.w3.org/TR/REC-html40/charset.html#h-5.2.2
 
 TODO
-====
-- parse @charset of HTML elements?
-- check for more texttypes if only text given
-    
+    - parse @charset of HTML elements?
+    - check for more texttypes if only text given
 """
 __all__ = ['buildlog',
            'encodingByMediaType',
@@ -94,22 +74,26 @@ __all__ = ['buildlog',
            'tryEncodings',
            'EncodingInfo']
 __docformat__ = 'restructuredtext'
-__author__ = 'Christof Hoeke'
-__version__ = '0.8.3 $Id: __init__.py 1143 2008-03-16 17:18:41Z cthedot $'
+__author__ = 'Christof Hoeke, Robert Siemer, Fredrik Hedman'
+__version__ = '$Id$'
 
-import cgi
 import HTMLParser
+import StringIO
+import cgi
 import httplib
 import re
-import StringIO
 import sys
 import types
 import urllib
 
+VERSION = '0.9.8'
+PY2x = sys.version_info < (3,0)
+
+
 class _MetaHTMLParser(HTMLParser.HTMLParser):
-    """parses given data for <meta http-equiv="content-type">"""
+    """Parse given data for <meta http-equiv="content-type">."""
     content_type = None
-    
+
     def handle_starttag(self, tag, attrs):
         if tag == 'meta' and not self.content_type:
             atts = dict([(a.lower(), v.lower()) for a, v in attrs])
@@ -137,37 +121,37 @@ _OTHER_TYPE = 4
 
 class EncodingInfo(object):
     """
-    All encoding related information, returned by ``getEncodingInfo``
-    
-    - ``encoding``: The guessed encoding
-        Encoding is the explicit or implicit encoding or None and
-        always lowercase.
+    All encoding related information, returned by :func:`getEncodingInfo`.
 
-    - from HTTP response    
-        * ``http_encoding``
-        * ``http_media_type``
+    Attributes filled:
+        - ``encoding``: The guessed encoding
+            Encoding is the explicit or implicit encoding or None and
+            always lowercase.
 
-    - from HTML <meta> element    
-        * ``meta_encoding``
-        * ``meta_media_type``
+        - from HTTP response
+            * ``http_encoding``
+            * ``http_media_type``
 
-    - from XML declaration
-        * ``xml_encoding``
+        - from HTML <meta> element
+            * ``meta_encoding``
+            * ``meta_media_type``
 
-    - ``mismatch``: True if mismatch between XML declaration and HTTP header
-        Mismatch is True if any mismatches between HTTP header, XML
-        declaration or textcontent (meta) are found. More detailed mismatch
-        reports are written to the optional log or ``logtext``
+        - from XML declaration
+            * ``xml_encoding``
 
-        Mismatches are not necessarily errors as preferences are defined.
-        For details see the specifications.
+        - ``mismatch``: True if mismatch between XML declaration and HTTP
+            header.
+            Mismatch is True if any mismatches between HTTP header, XML
+            declaration or textcontent (meta) are found. More detailed
+            mismatch reports are written to the optional log or ``logtext``
 
-    - ``logtext``: if no log was given log reports are given here 
-    
+            Mismatches are not necessarily errors as preferences are defined.
+            For details see the specifications.
+
+        - ``logtext``: if no log was given log reports are given here
     """
     def __init__(self):
-        """
-        initializes all possible properties to ``None``, see class
+        """Initialize all possible properties to ``None``, see class
         description
         """
         self.encoding = self.mismatch = self.logtext =\
@@ -175,61 +159,60 @@ class EncodingInfo(object):
             self.meta_encoding = self.meta_media_type =\
             self.xml_encoding =\
                 None
-        
+
     def __str__(self):
-        """
-        ``str(EncodingInfo())`` outputs the guessed encoding itself or the empty string
-        """
+        """Output the guessed encoding itself or the empty string."""
         if self.encoding:
             return self.encoding
         else:
             return u''
-            
+
     def __repr__(self):
         return "<%s.%s object encoding=%r mismatch=%s at 0x%x>" % (
-                self.__class__.__module__, self.__class__.__name__, 
+                self.__class__.__module__, self.__class__.__name__,
                 self.encoding, self.mismatch, id(self))
 
 
 def buildlog(logname='encutils', level='INFO', stream=sys.stderr,
             filename=None, filemode="w",
             format='%(levelname)s\t%(message)s'):
-    """
-    helper to build a basic log
-    
-    - if ``filename`` is given returns a log logging to ``filename`` with
-      mode ``filemode``
-    - else uses a log streaming to ``stream`` which defaults to
-      ``sys.stderr``
-    - ``level`` defines the level of the log
-    - ``format`` defines the formatter format of the log
+    """Helper to build a basic log
 
-    returns a log with the name ``logname``
+    - if `filename` is given returns a log logging to `filename` with
+      mode `filemode`
+    - else uses a log streaming to `stream` which defaults to
+      `sys.stderr`
+    - `level` defines the level of the log
+    - `format` defines the formatter format of the log
+
+    :returns:
+        a log with the name `logname`
     """
     import logging
-    
+
     log = logging.getLogger(logname)
-    
+
     if filename:
         hdlr = logging.FileHandler(filename, filemode)
     else:
         hdlr = logging.StreamHandler(stream)
-        
+
     formatter = logging.Formatter(format)
     hdlr.setFormatter(formatter)
 
     log.addHandler(hdlr)
     log.setLevel(logging.__dict__.get(level, logging.INFO))
 
-    return log    
+    return log
+
 
 def _getTextTypeByMediaType(media_type, log=None):
     """
-    returns type as defined by constants above
+    :returns:
+        type as defined by constants in this class
     """
     if not media_type:
         return _OTHER_TYPE
-    
     xml_application_types = [
         ur'application/.*?\+xml',
         u'application/xml',
@@ -239,7 +222,7 @@ def _getTextTypeByMediaType(media_type, log=None):
         ur'text\/.*?\+xml',
         u'text/xml',
         u'text/xml-external-parsed-entity']
-    
+
     media_type = media_type.strip().lower()
 
     if media_type in xml_application_types or\
@@ -255,26 +238,30 @@ def _getTextTypeByMediaType(media_type, log=None):
     elif media_type.startswith(u'text/'):
         return _TEXT_TYPE
     else:
-        return _OTHER_TYPE    
+        return _OTHER_TYPE
+
 
 def _getTextType(text, log=None):
-    """
-    checks if given text is XML (**naive test!**)
-    used if no content-type given    
+    """Check if given text is XML (**naive test!**)
+    used if no content-type given
     """
     if text[:30].find(u'<?xml version=') != -1:
         return _XML_APPLICATION_TYPE
     else:
         return _OTHER_TYPE
-    
+
+
 def encodingByMediaType(media_type, log=None):
     """
-    Returns a default encoding for the given media_type.
-    For example ``'utf-8'`` for ``media_type='application/xml'``. 
+    :param media_type:
+        a media type like "text/html"
+    :returns:
+        a default encoding for given `media_type`. For example
+        ``"utf-8"`` for ``media_type="application/xml"``.
 
-    Refers to RFC 3023 and HTTP MIME specification.
-   
-    If no default encoding is available returns ``None``.
+        If no default encoding is available returns ``None``.
+
+        Refers to RFC 3023 and HTTP MIME specification.
     """
     defaultencodings = {
         _XML_APPLICATION_TYPE: u'utf-8',
@@ -297,16 +284,22 @@ def encodingByMediaType(media_type, log=None):
                 media_type, encoding)
     return encoding
 
+
 def getHTTPInfo(response, log=None):
     """
-    Returns ``(media_type, encoding)`` information from the response'
-    Content-Type HTTP header. (Case of headers is ignored.)
-    May be ``(None, None)`` e.g. if no Content-Type header is
-    available.
+    :param response:
+        a HTTP response object
+    :returns:
+        ``(media_type, encoding)`` information from the `response`
+        Content-Type HTTP header. (Case of headers is ignored.)
+        May be ``(None, None)`` e.g. if no Content-Type header is
+        available.
     """
     info = response.info()
-    media_type = info.gettype()
-    encoding = info.getparam('charset') 
+    if PY2x:
+        media_type, encoding = info.gettype(), info.getparam('charset')
+    else:
+        media_type, encoding = info.get_content_type(), info.get_content_charset()
 
     if encoding:
         encoding = encoding.lower()
@@ -317,17 +310,27 @@ def getHTTPInfo(response, log=None):
 
     return media_type, encoding
 
+
 def getMetaInfo(text, log=None):
     """
-    Returns (media_type, encoding) information from (first)
-    X/HTML Content-Type ``<meta>`` element if available.
+    :param text:
+        a byte string
+    :returns:
+        ``(media_type, encoding)`` information from (first)
+        X/HTML Content-Type ``<meta>`` element if available in `text`.
 
-    Normally in X/HTML:
-        ``<meta http-equiv="Content-Type" content="media_type;
-        charset=encoding"/>``
+        XHTML format::
+
+            <meta http-equiv="Content-Type"
+                  content="media_type;charset=encoding" />
     """
     p = _MetaHTMLParser()
-    p.feed(text)
+
+    try:
+        p.feed(text)
+    except HTMLParser.HTMLParseError, e:
+        pass
+
     if p.content_type:
         media_type, params = cgi.parse_header(p.content_type)
         encoding = params.get('charset') # defaults to None
@@ -341,13 +344,18 @@ def getMetaInfo(text, log=None):
 
     return media_type, encoding
 
-def detectXMLEncoding(fp, log=None, includeDefault=True):
-    """
-    Attempts to detect the character encoding of the xml file
-    given by a file object fp. fp must not be a codec wrapped file
-    object! fp may also be a string or unicode string
 
-    The return value can be:
+def detectXMLEncoding(fp, log=None, includeDefault=True):
+    """Attempt to detect the character encoding of the xml file
+    given by a file object `fp`. `fp` must not be a codec wrapped file
+    object! `fp` may be a string or unicode string though.
+
+    Based on a recipe by Lars Tiede:
+    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/363841
+    which itself is based on Paul Prescotts recipe:
+    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/52257
+
+    :returns:
         - if detection of the BOM succeeds, the codec name of the
           corresponding unicode charset is returned
 
@@ -358,23 +366,20 @@ def detectXMLEncoding(fp, log=None, includeDefault=True):
 
         - if BOM and xml declaration fail, utf-8 is returned according
           to XML 1.0.
-
-    Based on a recipe by Lars Tiede:
-    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/363841
-    which itself is based on Paul Prescotts recipe:
-    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/52257
     """
-    if type(fp) in types.StringTypes:
+    if PY2x and isinstance(fp, types.StringTypes):
         fp = StringIO.StringIO(fp)
-    
+    elif isinstance(fp, (str,)):
+        fp = StringIO.StringIO(fp)
+
     ### detection using BOM
-    
+
     ## the BOMs we know, by their pattern
-    bomDict={ # bytepattern: name              
-             (0x00, 0x00, 0xFE, 0xFF) : "utf_32_be",        
+    bomDict={ # bytepattern: name
+             (0x00, 0x00, 0xFE, 0xFF) : "utf_32_be",
              (0xFF, 0xFE, 0x00, 0x00) : "utf_32_le",
-             (0xFE, 0xFF, None, None) : "utf_16_be", 
-             (0xFF, 0xFE, None, None) : "utf_16_le", 
+             (0xFE, 0xFF, None, None) : "utf_16_be",
+             (0xFF, 0xFE, None, None) : "utf_16_le",
              (0xEF, 0xBB, 0xBF, None) : "utf-8",
             }
 
@@ -400,7 +405,7 @@ def detectXMLEncoding(fp, log=None, includeDefault=True):
     ## still here? BOM detection failed.
     ##  now that BOM detection has failed we assume one byte character
     ##  encoding behaving ASCII
-    
+
     ### search xml declaration for encoding attribute
 
     ## assume xml declaration fits into the first 2 KB (*cough*)
@@ -438,19 +443,24 @@ def detectXMLEncoding(fp, log=None, includeDefault=True):
         else:
             return None
 
+
 def tryEncodings(text, log=None):
-    """
-    If installed uses chardet http://chardet.feedparser.org/ to detect
-    encoding, else tries different encodings on text and returns the one
+    """If installed uses chardet http://chardet.feedparser.org/ to detect
+    encoding, else tries different encodings on `text` and returns the one
     that does not raise an exception which is not very advanced or may
-    be totally wrong.
+    be totally wrong. The tried encoding are in order 'ascii', 'iso-8859-1',
+    'windows-1252' (which probably will never happen as 'iso-8859-1' can decode
+    these strings too) and lastly 'utf-8'.
 
-    Returns working encoding or None if no encoding does work at all.
+    :param text:
+        a byte string
+    :returns:
+        Working encoding or ``None`` if no encoding does work at all.
 
-    The returned encoding might nevertheless be not the one intended by the
-    author as it is only checked if the text might be encoded in that
-    encoding. Some texts might be working in "iso-8859-1" *and*
-    "windows-1252" *and* "ascii" *and* "utf-8" and ...
+        The returned encoding might nevertheless be not the one intended by
+        the author as it is only checked if the text might be encoded in
+        that encoding. Some texts might be working in "iso-8859-1" *and*
+        "windows-1252" *and* "ascii" *and* "utf-8" and ...
     """
     try:
         import chardet
@@ -462,119 +472,134 @@ def tryEncodings(text, log=None):
             log.warn(msg)
         else:
             print msg
-        
+
         encodings = (
             'ascii',
             'iso-8859-1',
-            'windows-1252',
+            #'windows-1252', # test later
             'utf-8'
             )
         encoding = None
         for e in encodings:
             try:
-                text.encode(e)
-            except (UnicodeEncodeError, UnicodeDecodeError):
+                text.decode(e)
+            except UnicodeDecodeError:
                 pass
             else:
-                encoding = e
-                break
+                if 'iso-8859-1' == e:
+                    try:
+                        if u'€' in text.decode('windows-1252'):
+                            return 'windows-1252'
+                    except UnicodeDecodeError:
+                        pass
+
+                return e
 
     return encoding
-            
+
+
 def getEncodingInfo(response=None, text=u'', log=None, url=None):
-    """
-    Finds all encoding related information in given ``text``.
-    Uses information in headers of supplied HTTPResponse, possible XML
-    declaration and X/HTML ``<meta>`` elements.
-    ``text`` will mostly be HTML or XML.
-    
-    Parameters
-        - ``response``: HTTP response object,
-          e.g. ``urllib.urlopen('url')``
-        - ``text``: to guess encoding for, might include XML
-          prolog with encoding pseudo attribute or HTML meta element 
-        - ``log``: an optional logging logger to which messages may go, if
-          no log given all log messages are available from resulting
-          ``EncodingInfo``
+    """Find all encoding related information in given `text`.
 
-    May also simply be called with ``getEncodingInfo(url='URL')`` which fetches
-    the url and all needed information.
+    Information in headers of supplied HTTPResponse, possible XML
+    declaration and X/HTML ``<meta>`` elements are used.
 
-    Returns instance of ``EncodingInfo``.
+    :param response:
+        HTTP response object, e.g. via ``urllib.urlopen('url')``
+    :param text:
+        a byte string to guess encoding for. XML prolog with
+        encoding pseudo attribute or HTML meta element will be used to detect
+        the encoding
+    :param url:
+        When given fetches document at `url` and all needed information.
+        No `reponse` or `text` parameters are needed in this case.
+    :param log:
+        an optional logging logger to which messages may go, if
+        no log given all log messages are available from resulting
+        ``EncodingInfo``
+
+    :returns:
+        instance of :class:`EncodingInfo`.
+
+    How the resulting encoding is retrieved:
 
-    How the resulting encoding is retrieved
-    =======================================
     XML
-    ---
-    RFC 3023 states if media type given in the Content-Type HTTP header is
-    application/xml, application/xml-dtd,
-    application/xml-external-parsed-entity, or any one of the subtypes of
-    application/xml such as application/atom+xml or application/rss+xml
-    etc then the character encoding is determined in this order:
+        RFC 3023 states if media type given in the Content-Type HTTP header is
+        application/xml, application/xml-dtd,
+        application/xml-external-parsed-entity, or any one of the subtypes of
+        application/xml such as application/atom+xml or application/rss+xml
+        etc then the character encoding is determined in this order:
 
-    1. the encoding given in the charset parameter of the Content-Type HTTP
-    header, or
-    2. the encoding given in the encoding attribute of the XML declaration
-    within the document, or
-    3. utf-8.
+        1. the encoding given in the charset parameter of the Content-Type HTTP
+        header, or
+        2. the encoding given in the encoding attribute of the XML declaration
+        within the document, or
+        3. utf-8.
 
-    Mismatch possibilities:
-        - HTTP + XMLdecla 
-        - HTTP + HTMLmeta
+        Mismatch possibilities:
+            - HTTP + XMLdecla
+            - HTTP + HTMLmeta
 
-        application/xhtml+xml ?
-            XMLdecla + HTMLmeta   
-        
-    If the media type given in the Content-Type HTTP header is text/xml,
-    text/xml-external-parsed-entity, or a subtype like text/Anything+xml,
-    the encoding attribute of the XML declaration is ignored completely
-    and the character encoding is determined in the order:
-    1. the encoding given in the charset parameter of the Content-Type HTTP
-    header, or
-    2. ascii.
+            application/xhtml+xml ?
+                XMLdecla + HTMLmeta
 
-    Mismatch possibilities:
-        - HTTP + XMLdecla 
-        - HTTP + HTMLmeta 
-        
-        text/xhtml+xml
-            XMLdecla + HTMLmeta 
+
+        If the media type given in the Content-Type HTTP header is text/xml,
+        text/xml-external-parsed-entity, or a subtype like text/Anything+xml,
+        the encoding attribute of the XML declaration is ignored completely
+        and the character encoding is determined in the order:
+        1. the encoding given in the charset parameter of the Content-Type HTTP
+        header, or
+        2. ascii.
+
+        No mismatch possible.
+
+
+        If no media type is given the XML encoding pseuso attribute is used
+        if present.
+
+        No mismatch possible.
 
     HTML
-    ----
-    For HTML served as text/html:
-        http://www.w3.org/TR/REC-html40/charset.html#h-5.2.2 
+        For HTML served as text/html:
+            http://www.w3.org/TR/REC-html40/charset.html#h-5.2.2
 
-    1. An HTTP "charset" parameter in a "Content-Type" field.
-       (maybe defaults to ISO-8859-1, but should not assume this)
-    2. A META declaration with "http-equiv" set to "Content-Type" and a
-       value set for "charset".
-    3. The charset attribute set on an element that designates an external
-       resource. (NOT IMPLEMENTED HERE YET)
+        1. An HTTP "charset" parameter in a "Content-Type" field.
+           (maybe defaults to ISO-8859-1, but should not assume this)
+        2. A META declaration with "http-equiv" set to "Content-Type" and a
+           value set for "charset".
+        3. The charset attribute set on an element that designates an external
+           resource. (NOT IMPLEMENTED HERE YET)
+
+        Mismatch possibilities:
+            - HTTP + HTMLmeta
 
-    Mismatch possibilities:
-        - HTTP + HTMLmeta 
-        
     TEXT
-    ----
-    For most text/* types the encoding will be reported as iso-8859-1. 
-    Exceptions are XML formats send as text/* mime type (see above) and 
-    text/css which has a default encoding of UTF-8. 
+        For most text/* types the encoding will be reported as iso-8859-1.
+        Exceptions are XML formats send as text/* mime type (see above) and
+        text/css which has a default encoding of UTF-8.
     """
     if url:
+        # may cause IOError which is raised
+        response = urllib.urlopen(url)
+
+    if text is None:
+        # read text from response only if not explicitly given
         try:
-            response = urllib.urlopen(url)
             text = response.read()
         except IOError, e:
-            print IOError(e)
-            sys.exit(1)
-        
+            pass
+
+    if text is None:
+        # text must be a string (not None)
+        text = ''
+
     encinfo = EncodingInfo()
 
     logstream = StringIO.StringIO()
     if not log:
         log = buildlog(stream=logstream, format='%(message)s')
-        
+
     # HTTP
     if response:
         encinfo.http_media_type, encinfo.http_encoding = getHTTPInfo(
@@ -584,13 +609,19 @@ def getEncodingInfo(response=None, text=u'', log=None, url=None):
         # check if maybe XML or (TODO:) HTML
         texttype = _getTextType(text, log)
 
-    # XML (also XHTML served as text/html)
-    if texttype == _XML_APPLICATION_TYPE or texttype == _XML_TEXT_TYPE:
-        encinfo.xml_encoding = detectXMLEncoding(text, log)
+    # XML only served as application/xml ! #(also XHTML served as text/html)
+    if texttype == _XML_APPLICATION_TYPE:# or texttype == _XML_TEXT_TYPE:
+        try:
+            encinfo.xml_encoding = detectXMLEncoding(text, log)
+        except (AttributeError, ValueError), e:
+            encinfo.xml_encoding = None
 
     # XML (also XHTML served as text/html)
     if texttype == _HTML_TEXT_TYPE:
-        encinfo.xml_encoding = detectXMLEncoding(text, log, includeDefault=False)
+        try:
+            encinfo.xml_encoding = detectXMLEncoding(text, log, includeDefault=False)
+        except (AttributeError, ValueError), e:
+            encinfo.xml_encoding = None
 
     # HTML
     if texttype == _HTML_TEXT_TYPE or texttype == _TEXT_TYPE:
@@ -607,7 +638,7 @@ def getEncodingInfo(response=None, text=u'', log=None, url=None):
     if texttype == _XML_APPLICATION_TYPE:
         if not encinfo.encoding:
             encinfo.encoding = encinfo.xml_encoding
-            # xml_encoding has default of utf-8            
+            # xml_encoding has default of utf-8
 
     #   text/html
     elif texttype == _HTML_TEXT_TYPE:
@@ -623,26 +654,30 @@ def getEncodingInfo(response=None, text=u'', log=None, url=None):
         if not encinfo.encoding:
             encinfo.encoding = encodingByMediaType(encinfo.http_media_type)
 
+    elif texttype == _TEXT_UTF8:
+        if not encinfo.encoding:
+            encinfo.encoding = encodingByMediaType(encinfo.http_media_type)
+
     # possible mismatches, checks if present at all and then if equal
     # HTTP + XML
     if encinfo.http_encoding and encinfo.xml_encoding and\
-       encinfo.http_encoding <> encinfo.xml_encoding:
+       encinfo.http_encoding != encinfo.xml_encoding:
         encinfo.mismatch = True
-        log.warn(u'"%s" (HTTP) <> "%s" (XML) encoding mismatch' %
+        log.warn(u'"%s" (HTTP) != "%s" (XML) encoding mismatch' %
                  (encinfo.http_encoding, encinfo.xml_encoding))
     # HTTP + Meta
     if encinfo.http_encoding and encinfo.meta_encoding and\
-         encinfo.http_encoding <> encinfo.meta_encoding:
+         encinfo.http_encoding != encinfo.meta_encoding:
         encinfo.mismatch = True
-        log.warn(u'"%s" (HTTP) <> "%s" (HTML <meta>) encoding mismatch' %
+        log.warn(u'"%s" (HTTP) != "%s" (HTML <meta>) encoding mismatch' %
                  (encinfo.http_encoding, encinfo.meta_encoding))
     # XML + Meta
     if encinfo.xml_encoding and encinfo.meta_encoding and\
-         encinfo.xml_encoding <> encinfo.meta_encoding:
+         encinfo.xml_encoding != encinfo.meta_encoding:
         encinfo.mismatch = True
-        log.warn(u'"%s" (XML) <> "%s" (HTML <meta>) encoding mismatch' %
+        log.warn(u'"%s" (XML) != "%s" (HTML <meta>) encoding mismatch' %
                  (encinfo.xml_encoding, encinfo.meta_encoding))
-                
+
     log.info(u'Encoding (probably): %s (Mismatch: %s)',
              encinfo.encoding, encinfo.mismatch)
 
